@@ -6,10 +6,11 @@
 #  See LICENSE for more info.
 
 import numpy as np
+from qtpy.QtGui import QMatrix4x4, QVector3D
 
 from OpenGL.GL import *
 from .gldrawable import GLDrawable
-from ..glprograms.textprogram import TextProgram
+from ..glprograms.text_management.textmanager import TextManager
 
 
 class TextGL(GLDrawable):
@@ -18,12 +19,31 @@ class TextGL(GLDrawable):
         self.text_vertices = []
         self.vertices = np.empty(3)
 
+        self._position = kwargs.get('position', np.zeros(3))
+        self._color = kwargs.get('color', np.ones(3))
+
         self.text = kwargs.get('text', ' ')
         self.scale = kwargs.get('scale', 1.0)
-        self.position = kwargs.get('position', [0.0, 0.0, 0.0])
-        self.orientation = kwargs.get('orientation', 'elevation')
+
+        self.rotation = kwargs.get('rotation', np.zeros(3))
         self.centered = kwargs.get('centered', False)
 
+    @property
+    def position(self) -> np.array:
+        return np.asarray(self._position, np.float32)
+
+    @position.setter
+    def position(self, value: iter) -> None:
+        self._position = np.array(value, np.float32)
+
+    @property
+    def color(self) -> np.array:
+        return np.asarray(self._color, np.float32)
+
+    @color.setter
+    def color(self, value: iter) -> None:
+        self._color = np.array(value, np.float32)
+
     def initialize(self) -> None:
         if self.is_initialized:
             return
@@ -42,37 +62,15 @@ class TextGL(GLDrawable):
     def tex_scale(self) -> float:
         return self.scale / 48
 
-    @staticmethod
-    def _rendering_buffer(xpos, ypos, zpos) -> np.ndarray:
-        return np.asarray(4 * [xpos, ypos, zpos], np.float32)
-
     @classmethod
-    def _rendering_buffer_elevation(cls, xpos, ypos, zpos, w, h) -> np.ndarray:
-        return cls._rendering_buffer(xpos, ypos, zpos) + np.asarray([
+    def _rendering_buffer(cls, xpos, ypos, zpos, w, h) -> np.ndarray:
+        return np.asarray(4 * [xpos, ypos, zpos], np.float32) + np.asarray([
             0, h, 0,
             0, 0, 0,
             w, h, 0,
             w, 0, 0,
         ], np.float32)
 
-    @classmethod
-    def _rendering_buffer_north(cls, xpos, ypos, zpos, w, h) -> np.ndarray:
-        return cls._rendering_buffer(xpos, ypos, zpos) + np.array([
-            0, 0, h,
-            0, 0, 0,
-            0, w, h,
-            0, w, 0,
-        ], np.float32)
-
-    @classmethod
-    def _rendering_buffer_east(cls, xpos, ypos, zpos, w, h) -> np.ndarray:
-        return cls._rendering_buffer(xpos, ypos, zpos) + np.asarray([
-            0, 0, h,
-            0, 0, 0,
-            w, 0, h,
-            w, 0, 0,
-        ], np.float32)
-
     @classmethod
     def _get_rendering_texture(cls) -> np.ndarray:
         return np.asarray([
@@ -83,51 +81,61 @@ class TextGL(GLDrawable):
         ], np.float32)
 
     def _setup_text_vertices(self) -> None:
-        self.text_vertices.clear()
+        text_vertices = []
 
-        # Retrieve positions
+        # Retrieve initial position
         x, y, z = self.position
 
+        # Setup text vertices
         for c in self.text:
-            ch = TextProgram.characters[c]
+            ch = TextManager.characters[c]
             w, h = ch.textureSize
             w = w * self.tex_scale
             h = h * self.tex_scale
 
-            # Select best vertices using orientation
-            if self.orientation == 'elevation':
-                self.text_vertices.append(self._rendering_buffer_elevation(x, y, z, w, h))
-                x += (ch.advance >> 6) * self.tex_scale
+            text_vertices.append(self._rendering_buffer(x, y, z, w, h))
+            x += (ch.advance >> 6) * self.tex_scale
 
-            elif self.orientation == 'north':
-                self.text_vertices.append(self._rendering_buffer_north(x, y, z, w, h))
-                y += (ch.advance >> 6) * self.tex_scale
+        tvs = np.array(text_vertices).reshape((-1, 3))
 
-            else:  # if self.orientation == 'east':
-                self.text_vertices.append(self._rendering_buffer_east(x, y, z, w, h))
-                x += (ch.advance >> 6) * self.tex_scale
+        # Rotate vertices as required
+        matrix = QMatrix4x4()
+
+        matrix.translate(*self.position)
+        matrix.rotate(self.rotation[0], 1.0, 0.0, 0.0)
+        matrix.rotate(self.rotation[1], 0.0, 1.0, 0.0)
+        matrix.rotate(self.rotation[2], 0.0, 0.0, 1.0)
+        matrix.translate(*-self.position)
+
+        tvs_rot3 = [matrix * QVector3D(*vec) for vec in tvs]
+        tvs = np.array([[vec.x(), vec.y(), vec.z()] for vec in tvs_rot3], np.float32).reshape((len(self.text), -1))
 
         # Centered = Position represents center of the whole text, instead of the bottom left.
         if self.centered:
-            tvs = np.array(self.text_vertices).reshape((-1, 3))
             ptp = np.ptp(tvs, axis=0)
             tvs -= ptp / 2
-            self.text_vertices = tvs.reshape((len(self.text), -1))
+
+        self.text_vertices = tvs.reshape((len(self.text), -1))
 
     def generate_buffers(self) -> None:
         self._vaos = [glGenVertexArrays(1)]
-        self._vbos = glGenBuffers(2)
+        self._vbos = glGenBuffers(3)
 
     def setup_attributes(self) -> None:
         _POSITION = 0
-        _TEXTURE = 1
+        _COLOR = 1
+        _TEXTURE = 2
 
         # Fill buffer (with empty data for now)
         glBindVertexArray(self.vao)
         self.fill_buffer(_POSITION, 3, np.zeros(4 * 3), GLfloat, GL_FLOAT, self._vbos[_POSITION])
+        self.fill_buffer(_COLOR, 3, self.color, GLfloat, GL_FLOAT, self._vbos[_COLOR])
         self.fill_buffer(_TEXTURE, 2, self._get_rendering_texture(), GLfloat, GL_FLOAT, self._vbos[_TEXTURE])
         glBindBuffer(GL_ARRAY_BUFFER, 0)
 
+        # Distribute along the whole instance
+        glVertexAttribDivisor(_COLOR, 1)
+
         glBindVertexArray(0)
 
     def draw(self) -> None:
@@ -135,7 +143,7 @@ class TextGL(GLDrawable):
         glBindVertexArray(self.vao)
 
         for i, c in enumerate(self.text):
-            ch = TextProgram.characters[c]
+            ch = TextManager.characters[c]
             vertices = self.text_vertices[i]
 
             # Render glyph texture over quad
