@@ -7,6 +7,8 @@
 
 import numpy as np
 
+from OpenGL.GL import *
+
 from .shaderprogram import ShaderProgram
 from .gridprogram import GridProgram
 from .textprogram import TextProgram
@@ -40,9 +42,8 @@ class GridComposite(ShaderProgram):
         text_drawables = []
 
         min_bound, max_bound = grid.bounding_box
-        max_width = max(map(lambda i: len(str(int(i))), grid.x_divisions.tolist() +
-                                                        grid.y_divisions.tolist() +
-                                                        grid.z_divisions.tolist()))
+        all_divisions = grid.x_divisions.tolist() + grid.y_divisions.tolist() + grid.z_divisions.tolist()
+        max_width = max(map(lambda i: len(str(int(i))), all_divisions))
 
         scale = grid.mark_separation / max_width
 
@@ -50,25 +51,49 @@ class GridComposite(ShaderProgram):
         # Remember to separate the number text from the grid
         for x in grid.x_divisions:
             pos = int(min_bound[0] + x)
-            textgl = TextGL(NullElement(), text=f'{pos}', scale=scale, centered=True, position=[pos,
-                                                                                                min_bound[1] - scale,
-                                                                                                min_bound[2]])
+            textgl = TextGL(NullElement(), text=f'{pos}', scale=scale, centered=False,
+                            rotation=[0.0, 0.0, 90.0], color=grid.text_color,
+                            position=[pos, min_bound[1], min_bound[2]])
+
+            # Ensure text doesn't overlap with grid
+            textgl.initialize()
+            tvs = textgl.text_vertices.reshape((-1, 3))
+            ptp = np.ptp(tvs, axis=0)
+            tvs -= [-ptp[0] / 2, ptp[1] + scale, 0.0]
+            textgl.text_vertices = tvs.reshape((len(textgl.text), -1))
+
             text_drawables.append(textgl)
 
         # Labels in Y
         for y in grid.y_divisions:
             pos = int(min_bound[1] + y)
-            textgl = TextGL(NullElement(), text=f'{pos}', scale=scale, centered=True, position=[min_bound[0] - scale,
-                                                                                                pos,
-                                                                                                min_bound[2]],)
+            textgl = TextGL(NullElement(), text=f'{pos}', scale=scale, centered=False,
+                            rotation=[0.0, 0.0, 0.0], color=grid.text_color,
+                            position=[min_bound[0], pos, min_bound[2]])
+
+            # Ensure text doesn't overlap with grid
+            textgl.initialize()
+            tvs = textgl.text_vertices.reshape((-1, 3))
+            ptp = np.ptp(tvs, axis=0)
+            tvs -= [ptp[0] + scale, ptp[1] / 2, 0.0]
+            textgl.text_vertices = tvs.reshape((len(textgl.text), -1))
+
             text_drawables.append(textgl)
 
         # Labels in Z
         for z in grid.z_divisions:
             pos = int(min_bound[2] + z)
-            textgl = TextGL(NullElement(), text=f'{pos}', scale=scale, centered=True, position=[min_bound[0] - 2 * scale,
-                                                                                                min_bound[1],
-                                                                                                pos])
+            textgl = TextGL(NullElement(), text=f'{pos}', scale=scale, centered=False,
+                            rotation=[90.0, 0.0, 0.0], color=grid.text_color,
+                            position=[min_bound[0], min_bound[1], pos])
+
+            # Ensure text doesn't overlap with grid
+            textgl.initialize()
+            tvs = textgl.text_vertices.reshape((-1, 3))
+            ptp = np.ptp(tvs, axis=0)
+            tvs -= [ptp[0] + scale, 0.0, ptp[2] / 2]
+            textgl.text_vertices = tvs.reshape((len(textgl.text), -1))
+
             text_drawables.append(textgl)
 
         return text_drawables
@@ -89,8 +114,12 @@ class GridComposite(ShaderProgram):
         pass
 
     def inner_draw(self, drawables: list) -> None:
+        glDisable(GL_DEPTH_TEST)
+
         self.grid_program.bind()
         self.grid_program.inner_draw(self.grid_program.drawables)
 
         self.text_program.bind()
         self.text_program.inner_draw(self.text_program.drawables)
+
+        glEnable(GL_DEPTH_TEST)
